{
  description = "Thorn Avery's 2023 Advent of Code";

  inputs = {
    nixpkgs.url = "github:NixOS/nixpkgs/nixos-23.05";
    crane = {
      url = "github:ipetkov/crane";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    flake-utils.url = "github:numtide/flake-utils";
    rust-overlay = {
      url = "github:oxalica/rust-overlay";
      inputs = {
        nixpkgs.follows = "nixpkgs";
        flake-utils.follows = "flake-utils";
      };
    };
    advisory-db = {
      url = "github:rustsec/advisory-db";
      flake = false;
    };
  };

  outputs = { self, nixpkgs, crane, flake-utils, advisory-db, rust-overlay, ... }:
    flake-utils.lib.eachDefaultSystem (system:
      let

        # ====================
        # == Customise Here ==
        # ====================

        version = "0.0.1";
        pname = "tAoC2023";

        # ====================
      
        pkgs = import nixpkgs {
          inherit system;
          overlays = [
            (import rust-overlay)
          ];
        };

        inherit (pkgs) lib;

        myRustTarget = pkgs.rust-bin.stable.latest.default;

        craneLib = (crane.mkLib pkgs).overrideToolchain myRustTarget;
        src = craneLib.cleanCargoSource (craneLib.path ./.);

        commonArgs = {
          inherit version pname;
          buildInputs = with pkgs; [
            pkg-config
            openssl
            cargo-mommy
            cargo-audit
            cargo-tarpaulin
            cargo-nextest
          ];
        };

        cargoArtifacts = craneLib.buildDepsOnly
          (commonArgs // { inherit src; });

        my-crate = craneLib.buildPackage
          (commonArgs // {
            inherit src cargoArtifacts;
          });
      in
      {
        checks = {
          inherit my-crate;

          my-crate-clippy = craneLib.cargoClippy (commonArgs // {
            inherit cargoArtifacts src;
            cargoClippyExtraArgs = "--all-targets -- --deny warnings";
          });

          my-crate-doc = craneLib.cargoDoc (commonArgs // {
            inherit cargoArtifacts src;
          });

          my-crate-fmt = craneLib.cargoFmt (commonArgs // {
            inherit src;
          });

          my-crate-audit = craneLib.cargoAudit (commonArgs // {
            inherit src advisory-db;
          });

          my-crate-nextest = craneLib.cargoNextest (commonArgs // {
            inherit cargoArtifacts src;
            partitions = 1;
            partitionType = "count";
          });
        } // lib.optionalAttrs (system == "x86_64-linux") {
          # cargo-tarpaulin only supports x86_64 systems
          my-crate-coverage = craneLib.cargoTarpaulin (commonArgs // {
            inherit cargoArtifacts src;
          });
        };

        packages.default = my-crate;

        apps.default = flake-utils.lib.mkApp {
          drv = my-crate;
        };

        devShells.default = pkgs.mkShell {
          inputsFrom = builtins.attrValues self.checks;
          RUST_SRC_PATH = "${pkgs.rust.packages.stable.rustPlatform.rustLibSrc}";
          nativeBuildInputs = with pkgs; [
            (pkgs.writeShellScriptBin "cargo" "${myRustTarget}/bin/cargo mommy $@")
            myRustTarget
          ] ++ commonArgs.buildInputs;
        };
      });
}
